package cz.muni.fi.pa165.books.web;

import cz.muni.fi.pa165.books.Book;
import cz.muni.fi.pa165.books.BookService;
import cz.muni.fi.pa165.books.validation.BookSpringValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.util.UriComponentsBuilder;

import javax.validation.Valid;
import java.util.List;
import java.util.Locale;

/**
 * SpringMVC Controller for handling books.
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
@Controller
@RequestMapping("/book")
public class BookController {

    final static Logger log = LoggerFactory.getLogger(BookController.class);

    @Autowired
    private BookService bookService;

    @Autowired
    private MessageSource messageSource;

    @ModelAttribute("books")
    public List<Book> allBook() {
        log.debug("allBooks()");
        return bookService.getAllBooks();
    }

    @ModelAttribute("colors")
    public Book.Color[] colors() {
        log.debug("colors()");
        return Book.Color.values();
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public String list(Model model) {
        log.debug("list()");
        model.addAttribute("book", new Book());
        return "book/list";
    }

    @RequestMapping(value = "/delete/{id}", method = RequestMethod.POST)
    public String delete(@PathVariable long id, RedirectAttributes redirectAttributes, Locale locale, UriComponentsBuilder uriBuilder) {
        log.debug("delete({})", id);
        Book book = bookService.getBook(id);
        bookService.deleteBook(book);
        redirectAttributes.addFlashAttribute(
                "message",
                messageSource.getMessage("book.delete.message", new Object[]{book.getTitle(), book.getAuthor(), book.getId()}, locale)
        );
        return "redirect:" + uriBuilder.path("/book/list").build();
    }

    @RequestMapping(value = "/update/{id}", method = RequestMethod.GET)
    public String update_form(@PathVariable long id, Model model) {
        Book book = bookService.getBook(id);
        model.addAttribute("book", book);
        log.debug("update_form(model={})", model);
        return "book/edit";
    }

    /**
     * Spring Validator added to JSR-303 Validator for this @Controller only
     * http://docs.spring.io/spring/docs/current/spring-framework-reference/html/validation.html#validation-mvc-configuring
     */
    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        binder.addValidators(new BookSpringValidator());
    }

    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public String update(@Valid @ModelAttribute Book book, BindingResult bindingResult, RedirectAttributes redirectAttributes, UriComponentsBuilder uriBuilder, Locale locale) {
        log.debug("update(locale={}, book={})", locale, book);
        if (bindingResult.hasErrors()) {
            log.debug("binding errors");
            for (ObjectError ge : bindingResult.getGlobalErrors()) {
                log.debug("ObjectError: {}", ge);
            }
            for (FieldError fe : bindingResult.getFieldErrors()) {
                log.debug("FieldError: {}", fe);
            }
            return book.getId()==null?"book/list":"book/edit";
        }
        if (book.getId() == null) {
            bookService.createBook(book);
            redirectAttributes.addFlashAttribute(
                    "message",
                    messageSource.getMessage("book.add.message", new Object[]{book.getTitle(), book.getAuthor(), book.getId()}, locale)
            );
        } else {
            bookService.updateBook(book);
            redirectAttributes.addFlashAttribute(
                    "message",
                    messageSource.getMessage("book.updated.message", new Object[]{book.getTitle(), book.getAuthor(), book.getId()}, locale)
            );
        }
        return "redirect:" + uriBuilder.path("/book/list").build();
    }

}
